@@ -29,7 +29,7 @@ class Expression:
 
         args = {
             k: ', '.join(
-                v.to_s(level + 1)
+                v.to_s(level + 1) if hasattr(v, 'to_s') else str(v)
                 for v in (vs if isinstance(vs, list) else [vs])
                 if v
             )
@@ -42,6 +42,11 @@ class Expression:
         return indent + left + right
 
 
+class CTE(Expression):
+    token_type = TokenType.WITH
+    arg_types = {'this': True, 'expressions': True}
+
+
 class Column(Expression):
     token_type = TokenType.COLUMN
     arg_types = {'this': True, 'db': False, 'table': False}
@@ -52,11 +57,6 @@ class Table(Expression):
     arg_types = {'this': True, 'db': False}
 
 
-class Select(Expression):
-    token_type = TokenType.SELECT
-    arg_types = {'expressions': True}
-
-
 class Group(Expression):
     token_type = TokenType.GROUP
     arg_types = {'this': True, 'expressions': True}
@@ -64,13 +64,28 @@ class Group(Expression):
 
 class Join(Expression):
     token_type = TokenType.JOIN
-    arg_types = {'this': True, 'expression': True, 'on': True, 'joiner': True}
+    arg_types = {'this': True, 'expression': True, 'on': True, 'side': False, 'kind': False}
 
 
 class Order(Expression):
     token_type = TokenType.ORDER
     arg_types = {'this': True, 'expressions': True, 'desc': False}
 
+
+class Union(Expression):
+    token_type = TokenType.UNION
+    arg_types = {'this': True, 'expression': True, 'distinct': True}
+
+
+class Select(Expression):
+    token_type = TokenType.SELECT
+    arg_types = {'expressions': True}
+
+
+class Window(Expression):
+    token_type = TokenType.OVER
+    arg_types = {'this': True, 'partition': False, 'order': False}
+
 # Binary Expressions
 # (PLUS a b)
 # (FROM table selects)
@@ -168,11 +183,22 @@ class Alias(Expression):
     token_type = TokenType.ALIAS
     arg_types = {'this': True, 'to': True}
 
+
+class Array(Expression):
+    token_type = TokenType.ARRAY
+    arg_types = {'expressions': True}
+
+
 class Between(Expression):
     token_type = TokenType.BETWEEN
     arg_types = {'this': True, 'low': True, 'high': True}
 
 
+class Bracket(Expression):
+    token_type = TokenType.BRACKET
+    arg_types = {'this': True, 'expressions': True}
+
+
 class Case(Expression):
     token_type = TokenType.CASE
     arg_types = {'ifs': True, 'default': False}
@@ -183,51 +209,34 @@ class Cast(Expression):
     arg_types = {'this': True, 'to': True}
 
 
+class Decimal(Expression):
+    token_type = TokenType.DECIMAL
+    arg_types = {'precision': False, 'scale': False}
+
+
+class Map(Expression):
+    token_type = TokenType.MAP
+    arg_types = {'keys': True, 'values': True}
+
+
 class In(Expression):
     token_type = TokenType.IN
     arg_types = {'this': True, 'expressions': True}
 
+
 # Functions
 class Func(Expression):
     token_type = TokenType.FUNC
-    arg_types = {'this': True}
+    arg_types = {'this': True, 'expressions': True}
 
 
-class Avg(Func):
-    pass
-
-
-class Coalesce(Func):
-    arg_types = {'expressions': True}
+class ApproxDistinct(Func):
+    arg_types = {'this': True, 'accuracy': False}
 
 
 class Count(Func):
     arg_types = {'this': False, 'distinct': False}
 
 
-class First(Func):
-    pass
-
-
-class Last(Func):
-    pass
-
-
 class If(Func):
     arg_types = {'condition': True, 'true': True, 'false': False}
-
-
-class LN(Func):
-    pass
-
-
-class Max(Func):
-    pass
-
-
-class Min(Func):
-    pass
-
-
-class Sum(Func):
-    pass
