@@ -1,9 +1,13 @@
-from enum import Enum, auto
+from enum import auto
 
-class TokenType(Enum):
+from sqlglot.helper import AutoName
+
+
+class TokenType(AutoName):
     PAREN = auto()
     L_PAREN = auto()
     R_PAREN = auto()
+    BRACKET = auto()
     L_BRACKET = auto()
     R_BRACKET = auto()
     L_BRACE = auto()
@@ -42,7 +46,7 @@ class TokenType(Enum):
     SMALLINT = auto()
     INT = auto()
     BIGINT = auto()
-    REAL = auto()
+    FLOAT = auto()
     DOUBLE = auto()
     DECIMAL = auto()
     CHAR = auto()
@@ -54,6 +58,7 @@ class TokenType(Enum):
     # keywords
     ALIAS = auto()
     ALL = auto()
+    ARRAY = auto()
     ASC = auto()
     BETWEEN = auto()
     BY = auto()
@@ -65,6 +70,7 @@ class TokenType(Enum):
     DISTINCT = auto()
     ELSE = auto()
     END = auto()
+    FULL = auto()
     FUNC = auto()
     FROM = auto()
     GROUP = auto()
@@ -74,9 +80,12 @@ class TokenType(Enum):
     IS = auto()
     JOIN = auto()
     LEFT = auto()
+    MAP = auto()
     NULL = auto()
     ON = auto()
     ORDER = auto()
+    OUTER = auto()
+    OVER = auto()
     PARTITION = auto()
     SELECT = auto()
     RIGHT = auto()
@@ -136,6 +145,7 @@ class Tokenizer:
         'DISTINCT': TokenType.DISTINCT,
         'ELSE': TokenType.ELSE,
         'END': TokenType.END,
+        'FULL': TokenType.FULL,
         'FROM': TokenType.FROM,
         'GROUP': TokenType.GROUP,
         'HAVING': TokenType.HAVING,
@@ -149,7 +159,8 @@ class Tokenizer:
         'ON': TokenType.ON,
         'OR': TokenType.OR,
         'ORDER': TokenType.ORDER,
-        'OVER': TokenType.BY,
+        'OUTER': TokenType.OUTER,
+        'OVER': TokenType.OVER,
         'PARTITION': TokenType.PARTITION,
         'RIGHT': TokenType.RIGHT,
         'SELECT': TokenType.SELECT,
@@ -157,18 +168,28 @@ class Tokenizer:
         'UNION': TokenType.UNION,
         'WHEN': TokenType.WHEN,
         'WHERE': TokenType.WHERE,
+        'WITH': TokenType.WITH,
 
+        'ARRAY': TokenType.ARRAY,
         'BOOL': TokenType.BOOLEAN,
         'BOOLEAN': TokenType.BOOLEAN,
+        'TINYINT': TokenType.TINYINT,
         'SMALLINT': TokenType.SMALLINT,
+        'INT2': TokenType.SMALLINT,
         'INTEGER': TokenType.INT,
         'INT': TokenType.INT,
         'INT4': TokenType.INT,
         'BIGINT': TokenType.BIGINT,
         'INT8': TokenType.BIGINT,
         'DECIMAL': TokenType.DECIMAL,
+        'MAP': TokenType.MAP,
         'NUMERIC': TokenType.DECIMAL,
-        'REAL': TokenType.REAL,
+        'DEC': TokenType.DECIMAL,
+        'FIXED': TokenType.DECIMAL,
+        'REAL': TokenType.FLOAT,
+        'FLOAT': TokenType.FLOAT,
+        'FLOAT4': TokenType.FLOAT,
+        'FLOAT8': TokenType.DOUBLE,
         'DOUBLE': TokenType.DOUBLE,
         'JSON': TokenType.JSON,
         'CHAR': TokenType.CHAR,
@@ -185,23 +206,26 @@ class Tokenizer:
         "\rn": TokenType.BREAK,
     }
 
-    def __init__(self, code, **kwargs):
-        self.code = code
+    def __init__(self, **opts):
+        self.quote = opts.get('quote') or "'"
+        self.identifier = opts.get('identifier') or '"'
+        self.single_tokens = {**self.SINGLE_TOKENS, **opts.get('single_tokens', {})}
+        self.keywords = {**self.KEYWORDS, **opts.get('keywords', {})}
+        self.white_space = {**self.WHITE_SPACE, **opts.get('white_space', {})}
+        self.reset()
+
+    def reset(self):
+        self.code = ''
         self.tokens = []
-
-        self.quote = kwargs.get('quote', "'")
-        self.identifier = kwargs.get('identifier', '"')
-        self.single_tokens = {**self.SINGLE_TOKENS, **kwargs.get('single_tokens', {})}
-        self.keywords = {**self.KEYWORDS, **kwargs.get('keywords', {})}
-        self.white_space = {**self.WHITE_SPACE, **kwargs.get('white_space', {})}
-
         self._start = 0
         self._current = 0
         self._line = 0
         self._col = 0
 
-    def tokenize(self):
-        # pylint: disable=too-many-branches
+    def tokenize(self, code): # pylint: disable=too-many-branches
+        self.reset()
+        self.code = code
+
         while self._current < len(self.code):
             self._start = self._current
             self._advance()
